/*----------------------------------------------------------------------------
 *      R T L   T C P N E T   E x a m p l e
 *----------------------------------------------------------------------------
 *      Name:    HTTP_DEMO.C
 *      Purpose: HTTP Server demo example
 *      Rev.:    V3.20
 *----------------------------------------------------------------------------
 *      This code is part of the RealView Run-Time Library.
 *      Copyright (c) 2004-2008 KEIL - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <RTL.h>
#include <Net_Config.h>
#include <AT91SAM7X256.H>                     /* AT91SAM7X256 definitions    */
#include "AT91SAM7X-EK.h"

/* Clock Definitions */
#define EXT_OSC 18432000                      /*  External Oscillator MAINCK */
#define MCK     47923200                      /*  Master Clock               */
#define BAUD(b) ((MCK + 8*b)/(16*b))
#define TCNT    (MCK/16)/10                   /* Timer Counts for 100ms      */

BOOL LEDrun;
BOOL tick;
U32  dhcp_tout;

extern LOCALM localm[];                       /* Local Machine Settings      */
#define MY_IP localm[NETIF_ETH].IpAdr
#define DHCP_TOUT   50                        /* DHCP timeout 5 seconds      */

static void init_io (void);
#ifdef RTX_KERNEL
 U64 tcp_stack[800/8];                        /* A bigger stack for tcp_task */

 /* Forward references */
 __task void init       (void);
 __task void blink_led  (void);
 __task void timer_task (void);
 __task void tcp_task   (void);
#endif

/*--------------------------- init ------------------------------------------*/

#ifndef RTX_KERNEL

static void init () {
   /* Add System initialisation code here */

   init_io ();
   init_TcpNet ();
   /* PITC Timer interval timer, reload to 100ms */
   *AT91C_PITC_PIMR = (TCNT - 1) | AT91C_PITC_PITEN;
}

#else

__task void init (void) {
   /* Add System initialisation code here */ 

   init_io ();
   init_TcpNet ();

   /* Initialize Tasks */
   os_tsk_prio_self (100);
   os_tsk_create (blink_led, 20);
   os_tsk_create (timer_task, 30);
   os_tsk_create_user (tcp_task, 0, &tcp_stack, sizeof(tcp_stack));
   os_tsk_delete_self();
}

#endif

/*--------------------------- timer_poll ------------------------------------*/

#ifndef RTX_KERNEL

static void timer_poll () {
   /* System tick timer running in poll mode */
   volatile U32 val;

   if (*AT91C_PITC_PISR & 1) {
      val = *AT91C_PITC_PIVR;
      /* Timer tick every 100 ms */
      timer_tick ();
      tick = __TRUE;
   }
}

#else

__task void timer_task (void) {
   /* System tick timer task */
   os_itv_set (10);
   while (1) {
      timer_tick ();
      tick = __TRUE;
      os_itv_wait ();
   }
}

#endif


/*--------------------------- init_io ---------------------------------------*/

static void init_io () {
   /* I/O Ports configured as Output. */
   *AT91C_PMC_PCER  = (1 << AT91C_ID_PIOB) | /* Enable Clock for PIO (LEDs)  */
                      (1 << AT91C_ID_PIOA);  /*                  (Joystick)  */
   *AT91C_PIOB_PER  = AT91B_LED_MASK;        /* Enable PIO for LED1..4       */
   *AT91C_PIOB_OER  = AT91B_LED_MASK;        /* LED1..4 are Outputs          */
   *AT91C_PIOB_SODR = AT91B_LED_MASK;        /* Turn off LED's ("1")         */

   /* Configure DBGU for 115200 baud. */
   *AT91C_PIOA_IDR  =
   *AT91C_PIOA_PPUDR=
   *AT91C_PIOA_ASR  =
   *AT91C_PIOA_PDR  = AT91C_PA27_DRXD | AT91C_PA28_DTXD;

   *AT91C_DBGU_CR   = AT91C_US_RSTRX   | AT91C_US_RSTTX;
   *AT91C_DBGU_IDR  = 0xFFFFFFFF;
   *AT91C_DBGU_BRGR = BAUD(115200);
   *AT91C_DBGU_MR   = AT91C_US_PAR_NONE;
   *AT91C_DBGU_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;
   *AT91C_DBGU_CR   = AT91C_US_RXEN    | AT91C_US_TXEN;
}


/*--------------------------- fputc -----------------------------------------*/

int fputc(int ch, FILE *f)  {
   /* Debug output to serial port. */

   if (ch == '\n')  {
      while (!(*AT91C_DBGU_CSR & AT91C_US_TXRDY));
      *AT91C_DBGU_THR = '\r';               /* output CR                    */
   }
   while (!(*AT91C_DBGU_CSR & AT91C_US_TXRDY));
   *AT91C_DBGU_THR = ch;
   return (ch);
}


/*--------------------------- LED_out ---------------------------------------*/

void LED_out (U32 val) {

   *AT91C_PIOB_SODR = AT91B_LED_MASK;
   if (val & 1) {
      *AT91C_PIOB_CODR = AT91B_LED4;         /* Turn LED4 On                 */
   }
   if (val & 2) {
      *AT91C_PIOB_CODR = AT91B_LED3;         /* Turn LED3 On                 */
   }
   if (val & 4) {
      *AT91C_PIOB_CODR = AT91B_LED2;         /* Turn LED2 On                 */
   }
   if (val & 8) {
      *AT91C_PIOB_CODR = AT91B_LED1;         /* Turn LED1 On                 */
   }
}


/*--------------------------- get_button ------------------------------------*/

U8 get_button (void) {
   /* Read ARM Digital Input */
   U32 key,val = 0;

   key = *AT91C_PIOA_PDSR;
   if ((key & AT91B_SW1) == 0) {
      /* Check if SW1 is pressed */
      val |= 0x01;
   }
   if ((key & AT91B_SW2) == 0) {
      /* Check if SW2 is pressed */
      val |= 0x02;
   }
   if ((key & AT91B_SW3) == 0) {
      /* Check if SW3 is pressed */
      val |= 0x04;
   }
   if ((key & AT91B_SW4) == 0) {
      /* Check if SW4 is pressed */
      val |= 0x08;
   }
   if ((key & AT91B_SW5) == 0) {
      /* Check if SW5 is pressed */
      val |= 0x10;
   }
   return (val);
}


/*--------------------------- dhcp_check ------------------------------------*/

static void dhcp_check () {
   /* Monitor DHCP IP address assignment. */

   if (tick == __FALSE || dhcp_tout == 0) {
      return;
   }
#ifdef RTX_KERNEL
   tick = __FALSE;
#endif
   if (mem_test (&MY_IP, 0, IP_ADRLEN) == __FALSE) {
      /* Success, DHCP has already got the IP address. */
      dhcp_tout = 0;
      return;
   }
   if (--dhcp_tout == 0) {
      /* A timeout, disable DHCP and use static IP address. */
      dhcp_disable ();
   }
}


/*--------------------------- blink_led -------------------------------------*/

#ifndef RTX_KERNEL

static void blink_led () {
   /* Blink the LEDs on AT91SAM7X board */
   const U8 led_val[8] = { 0x01,0x03,0x07,0x0F,0x0E,0x0C,0x08,0x00 };
   static U32 cnt;

   if (tick == __TRUE) {
      /* Every 100 ms */
      tick = __FALSE;
      if (LEDrun == __TRUE) {
         LED_out (led_val[cnt]);
         if (++cnt >= sizeof(led_val)) {
            cnt = 0;
         }
      }
   }
}

#else

__task void blink_led () {
   /* Blink the LEDs on AT91SAM7X board */
   const U8 led_val[8] = { 0x01,0x03,0x07,0x0F,0x0E,0x0C,0x08,0x00 };
   U32 cnt = 0;

   LEDrun = __TRUE;
   while(1) {
      /* Every 100 ms */
      if (LEDrun == __TRUE) {
         LED_out (led_val[cnt]);
         if (++cnt >= sizeof(led_val)) {
            cnt = 0;
         }
      }
      os_dly_wait(10);
   }
}

#endif


/*---------------------------------------------------------------------------*/

#ifndef RTX_KERNEL

int main (void) {
   /* Main Thread of the TcpNet */

   init ();
   LEDrun = __TRUE;
   dhcp_tout = DHCP_TOUT;
   while (1) {
      timer_poll ();
      main_TcpNet ();
      dhcp_check ();
      blink_led ();
   }
}

#else

__task void tcp_task (void) {
   /* Main Thread of the TcpNet. This task should have */
   /* the lowest priority because it is always READY. */
   dhcp_tout = DHCP_TOUT;
   while (1) {
      main_TcpNet();
      dhcp_check ();
      os_tsk_pass();
   }
}


int main (void) {
   /* Start with 'init' task. */
   os_sys_init(init);
   while(1);
}

#endif


/*----------------------------------------------------------------------------
 * end of file
 *---------------------------------------------------------------------------*/


